
/*
 * Model of Discriminatory Altruism
 * (variant of Ethno model)
 *
 * Version m3.1.5
 *
 * This code created originally on 1/21/03
 * (copied from package "ethnoadd" and modified)
 *
 * Additions:
 *
 * 1/31/03 fixes for:
 *		- correction in genotype summation fencepost problem
 *		- "blank" agents in lattice view
 *		- timeseries update timing (need to exclude "culled" agents)
 *
 * 2/4/03 additions for regression data
 *
 * 2/5/03 changes:
 *		  - changed output format for regression
 *		  - fixed code for count of colors
 *		  - addition of some batch capabilities
 *		  - clean up some of testing code
 *
 * 2/7/03 changes:
 *		- add StatCollectors to count strategy/color combinations
 *
 * 2/10/03 changes: (3.1.4)
 *		- add "shadow" and "founder" alleles
 *		- add agent color to regression data output
 *		- add PTRi to regression data output
 *
 * 5/15/03 changes: (3.1.5)
 *		- fix last100 genotype counters
 *      - cosmetical fixes of output files
 *      - updated random interaction code (now 4 interactions per agent)
 *
 */

package edu.brook.ethno10;

//import java.lang.*;
import java.awt.*;
import java.io.*;
import java.util.*;
import java.text.*;

import edu.brook.ascape.model.*;
import edu.brook.ascape.util.*;
import edu.brook.ascape.view.*;

import edu.brook.*;

public class EthnoModel extends ScapeVector {


//USER-MODIFIABLE:
//----------------------------------------------------------------------              
    //model parameters
    protected int latticeWidth = 50; //50 for "base" case
    protected int latticeHeight = 50; //50 for "base" case
	protected double baseOTR = 0.12; //0.12 "base" case
    protected double costOfGiving = .01; //.01 "base" case
    protected double gainOfReceiving = .03; //.03 "base" case
	protected double mutationRate = 0.001; //.001 "base" case
    protected int colorSetting = 4; //must be 1,2,4,6,8 etc.
    protected int stopAtIteration = 2000; //2000 "base" case
    protected double vacancyProportion = 0.20; //0.2 for "base" case
    
    //runtime control settings
    protected boolean restartModel = false; //true
	protected boolean useSpecifiedRandomSeed = true; //false
    protected long specifiedSeed = 1053216937313L; //1053216937313L

    //model variant switches
	protected boolean combatOn = false; //(see Eth1001 memo)
	protected boolean discrimCostOn = false;
	protected double discrimCost = 0.00;	
    protected int childNearParentZeroFalse = 2; //2 "base case; anything other than 0 means true
    protected boolean locateChildNearParent = true; //true for "base case"
    protected boolean noiseOn = false;  //false for "base" case
    protected double noiseLevel = 0;

    //data output switches
    protected boolean completeDataOutput = true; //iter by iter data for a run
    protected boolean summaryDataOutput = true; //summary aggregate data only
    protected boolean headerData = true; //header info for run (parameter settings, etc.)
    protected boolean regressionData = true; //indiv data for regressions (last period only)
    protected boolean viewsOn = false;
    protected boolean sweepIsOn = false;
    protected String outputPath = "D:/Daten Axelrod/Data/"; // Ross: "c:/Docs, Data, Files/Ethnocentrism Model/Data and Analysis/model 1.0/"  Christian: "D:/Daten Axelrod/Data/"    
    
    //Experiment Settings
    protected int experimentNumber = 2; //experiment number for run(s)
    protected String experimentDescr = "test for debugging"; //short exp description
    protected int runMax = 1; //number of runs for experiment
	    
//--------------------------------------------------------------------------------------    
    
//INTERNAL USE -- DO NOT MODIFY
   
    public boolean verbose = false;  //for testing
    public boolean verbosePlacement = false; //for testing
   
   	public boolean regressionTest = false;
   	public boolean founderDataOutput = false;
   
   	public String versionNumber = "3.1.5";
	public String delimiter = "	";   	
    
    //constants
    public static final boolean COOPERATE = true;
    public static final boolean DEFECT = false;
    public static final int BLUE = 0;
    public static final int RED = 1;

    protected boolean useMoreColors = true; //default setting, adjusted by "# of colors" variable
    protected boolean useOneTagOnly = false; //"    "
     
    protected boolean localInteraction = true;
     
	//internal counters
    protected int[] phenotypeStats = new int[5]; //meet own, meet other, def other, coop own
	protected double[] lasthundredPhenotypeStats = new double[5];
	protected double[] lasthundredGenotypeStats = new double[4];
	protected final int MEETOWN = 0;
	protected final int MEETOTHER = 1;
	protected final int DEFOTHER = 2;
	protected final int COOPOWN = 3;
	protected final int COOPOTHER = 4;
	protected final int CC = 0;
	protected final int CD = 1;
	protected final int DC = 2;
	protected final int DD = 3;
	
    protected int playMode = 0;

	public int runNumber;
	
    ScapeGraph lattice;
    
    ScapeVector agents;
	ScapeVector unplacedClones;
    
    //Vector agentRegressionList;
    
    Overhead2DView overheadView;
    
    StatCollectorCond countEthnocentrics;
    StatCollectorCond countTotalDefectors;
    StatCollectorCond countRed;
    StatCollectorCond countBlue;
    StatCollectorCond countGreen;
    StatCollectorCond countYellow;
    StatCollectorCond countRedEthnos;
    StatCollectorCond countBlueEthnos;
    StatCollectorCond countGreenEthnos;
    StatCollectorCond countYellowEthnos;
    StatCollectorCond countCoopBoth;
    StatCollectorCond countCoopOwnDefOther;
    StatCollectorCond countDefOwnCoopOther;
    StatCollectorCond countDefBoth;
    
    //These are the file names and paths for iter-by-iter data output
    File outputfile = new File(outputPath+"expL"+experimentNumber+" batchData.txt");
	File outputfile2 = new File(outputPath+"expL"+experimentNumber+" batchRunFile.txt");
	
	//This is file name and path for Summary data output
	//A header file named "about.txt" will also be created in this directory
	//containing the parameters used for all the experiment runs
	String summaryDataFile = (outputPath+"expL"+experimentNumber+" summary data.txt");
	String regressionDataFile = (outputPath+"expL"+experimentNumber+" regr data");                  
    
    ChartView chartView;
    ChartView behaviorChart;
    
    public void createScape() { //this initializes memory for scape vectors, etc.
        super.createScape();
        
        //This sets up a lattice (using Von N geometry) of size as specified
        //above, and fills it with cells of type "EthnoCell" to host players 
        lattice = new ScapeArray2DVonNeumann(); //VonNeumann()
        lattice.setPrototypeAgent(new EthnoCell());
        lattice.setExtent(new Coordinate2DDiscrete(latticeWidth, latticeHeight));
        
        //This sets up a population of players, who live on the lattice
        EthnoPlayer player = new edu.brook.ethno10.EthnoPlayer();
        player.setHostScape(lattice);
		agents = new ScapeVector();
		agents.setName("Players");
		agents.setPrototypeAgent(player);
        agents.setExecutionOrder(Scape.RULE_ORDER);
				
		addAgent(lattice);
		addAgent(agents);
	
		unplacedClones = new ScapeVector();
		
		//This code instructs the model to stop automatically at the specified iteration
		try {
    		setStopPeriod(stopAtIteration);
    	}
    	catch (SpatialTemporalException e) {
    	    System.out.println("Bad start/stop periods: " + e);
    	}
	
	    //This code instructs the model to restart automatically if the number of
	    //runs completed so far is less than the desired total
	    setAutoRestart(restartModel);
	    
	    //For specifying a random seed
	    if (useSpecifiedRandomSeed) {
	        setRandomSeed(specifiedSeed);
	    }
	    
	    //This code initializes basic statistics counters
	    for (int i=0;i<phenotypeStats.length;i++) {
	        phenotypeStats[i] = 0;
	        lasthundredPhenotypeStats[i] = 0;
	    }
	    
	    for (int i=0;i<4;i++) {
	        lasthundredGenotypeStats[i] = 0;
	    }
	    
	    /*
	    if (regressionData) {
	    	agentRegressionList = new Vector();
	    }
	    */
	    
	   // runNumber = 0;
	    
	    //This code specifies the output syntax for the header file
	    if (headerData) {
	     try {
      	        FileOutputStream fos = new FileOutputStream(outputPath+"expL"+experimentNumber+" about.txt", false);		  
                PrintWriter out = new PrintWriter(fos);
                out.print("EXP");
                out.println(experimentNumber);
                Date currentTime = new Date();
                SimpleDateFormat sdf = new SimpleDateFormat();
                out.println(sdf.format(currentTime));
	            out.println("------------------------------");                
                out.println(experimentDescr);
                out.println("Code version " + versionNumber);
                out.println("------------------------------");
                
                out.print("latticeWidth ");
                out.print(delimiter);
                out.println(latticeWidth);
            
                out.print("latticeHeight ");
                out.print(delimiter);
                out.println(latticeHeight);
  	        
  	            out.print("baseOTR ");
                out.print(delimiter);
                out.println(baseOTR);;
            
                out.print("costOfGiving ");
                out.print(delimiter);
                out.println(costOfGiving);
            
                out.print("benefitofReceiving ");
                out.print(delimiter);
                out.println(gainOfReceiving);
            
                out.print("mutationRate ");
                out.print(delimiter);
                out.println(mutationRate);
    
                out.print("colorSetting ");
                out.print(delimiter);
                out.println(colorSetting);
    			out.print("stopAtIteration ");
    		    out.print(delimiter);
            	out.println(stopAtIteration);            
				out.print("vacancyProportion ");
    		    out.print(delimiter);
            	out.println(vacancyProportion);            
                                                      
                out.print("offspring placement ");
                out.print(delimiter);
                if (locateChildNearParent) {out.println("LOCAL");}
                else {out.println("RANDOM");}

  				out.print("interaction topology ");
                if (getLocalInteraction()) {out.println("LOCAL");}
                else {out.println("RANDOM");}
                out.print("combat switch ");
                out.print(delimiter);
                if (combatOn) {out.println("ON");}
                else {out.println("OFF");}
    			
    			out.print("discrim cost ");
    		    out.print(delimiter);
            	if (discrimCostOn) {
    				out.print("ON - level: ");
    				out.println(discrimCost);
    				}
                else {out.println("OFF");}
    			            
    			out.print("noise switch ");
                out.print(delimiter);
            	if (noiseOn) {
    				out.print("ON - level: ");
    				out.println(noiseLevel);
    				}
                else {out.println("OFF");}
    	
	            out.close();
        	    } catch (FileNotFoundException e) {
		        	System.err.println("FileStreamsTest: "+ e);
	        	} catch (IOException e) {
			        System.err.println("FileStreamsTest: "+ e);
		        }	
		 
	    }
	    
	}
		
	public void createViews() {
	    super.createViews();
	    
	    
	    //STATISTICS COLLECTORS
	    countEthnocentrics = new StatCollectorCond("Ethnocentrics", false) {
                public boolean meetsCondition(Object object) {
                   return ((((EthnoPlayer) object).strategyToOwn) && (!((EthnoPlayer) object).strategyToOther));              
                }
           };
	    
	    countTotalDefectors = new StatCollectorCond("DefectAgainstAll", false) {
	            public boolean meetsCondition(Object object) {
	                return ((!((EthnoPlayer) object).strategyToOwn) && (!((EthnoPlayer) object).strategyToOther));
	            }
	            
	    };
	    
	    countRed = new StatCollectorCond("Reds", false) {
	            public boolean meetsCondition(Object object) {
	                return (((EthnoPlayer) object).tag == 1);
	            }
	    };
	    
	    countBlue = new StatCollectorCond("Blues", false) {
	            public boolean meetsCondition(Object object) {
	                return (((EthnoPlayer) object).tag == 0);
	            }
	    };
	    
	    countGreen = new StatCollectorCond("Greens", false) {
	            public boolean meetsCondition(Object object) {
	                return (((EthnoPlayer) object).tag == 2);
	            }
	    };
	    
	    countYellow = new StatCollectorCond("Yellows", false) {
	            public boolean meetsCondition(Object object) {
	                return (((EthnoPlayer) object).tag == 3);
	            }
	    };
	    

		countRedEthnos = new StatCollectorCond("RedDiscrim", false) {
		        public boolean meetsCondition(Object object) {
	                return ((((EthnoPlayer) object).tag == 1) && (((EthnoPlayer) object).strategyToOwn) && (!(((EthnoPlayer) object).strategyToOther)));
	            }			
		};
	    
	    countBlueEthnos = new StatCollectorCond("BlueDiscrim", false) {
		        public boolean meetsCondition(Object object) {
	                return ((((EthnoPlayer) object).tag == 0) && (((EthnoPlayer) object).strategyToOwn) && (!(((EthnoPlayer) object).strategyToOther)));
	            }			
		};
	    
	    countGreenEthnos = new StatCollectorCond("PurpleDiscrim", false) {
		        public boolean meetsCondition(Object object) {
	                return ((((EthnoPlayer) object).tag == 2) && (((EthnoPlayer) object).strategyToOwn) && (!(((EthnoPlayer) object).strategyToOther)));
	            }			
		};
	    
	    countYellowEthnos = new StatCollectorCond("YellowDiscrim", false) {
		        public boolean meetsCondition(Object object) {
	                return ((((EthnoPlayer) object).tag == 3) && (((EthnoPlayer) object).strategyToOwn) && (!(((EthnoPlayer) object).strategyToOther)));
	            }			
		};
	    
	    
	    //GENOTYPE STATS
	    
  	    countCoopBoth = new StatCollectorCond("CoopBoth", false) {
	            public boolean meetsCondition(Object object) {
	                return ((((EthnoPlayer) object).strategyToOwn) && ((EthnoPlayer) object).strategyToOther);
	            }
	    };
	    countCoopOwnDefOther = new StatCollectorCond("CoopOwnDefOther", false) {
	            public boolean meetsCondition(Object object) {
   	                return ((((EthnoPlayer) object).strategyToOwn) && !((EthnoPlayer) object).strategyToOther);
	            }
	    };
	    countDefOwnCoopOther = new StatCollectorCond("DefOwnCoopOther", false) {
	            public boolean meetsCondition(Object object) {
                    return (!(((EthnoPlayer) object).strategyToOwn) && ((EthnoPlayer) object).strategyToOther);
	            }
	    };
	    countDefBoth = new StatCollectorCond("DefBoth", false) {
	            public boolean meetsCondition(Object object) {
	                return (!(((EthnoPlayer) object).strategyToOwn) && !((EthnoPlayer) object).strategyToOther);
	            }
	    };

        
	    agents.addStatCollector(countEthnocentrics);	
	    agents.addStatCollector(countTotalDefectors);
	    agents.addStatCollector(countRed);
	    agents.addStatCollector(countBlue);
	    agents.addStatCollector(countGreen);
	    agents.addStatCollector(countYellow);
	    agents.addStatCollector(countRedEthnos);
	    agents.addStatCollector(countBlueEthnos);
	    agents.addStatCollector(countGreenEthnos);
	    agents.addStatCollector(countYellowEthnos);
	    agents.addStatCollector(countCoopBoth);
	    agents.addStatCollector(countCoopOwnDefOther);
	    agents.addStatCollector(countDefOwnCoopOther);
	    agents.addStatCollector(countDefBoth);
	 
	 
	 	if (sweepIsOn) {
	 
	 	SweepControlView ethnoSweeper = new SweepControlView();
        addView(ethnoSweeper);

        SweepLink colorSweep = new SweepLink();
        colorSweep.addMember(new SweepDimension(this, "CostOfGiving", 1.0, 2.0, 1.0));
       
        ethnoSweeper.getSweepGroup().addMember(colorSweep);
        ethnoSweeper.getSweepGroup().setRunsPer(5);
        
    	}
           
	    
	    //SET UP BASIC ITERATION BY ITERATION DATA OUTPUT
	    if (completeDataOutput) {
	        DataOutputView dataRecord = new DataOutputView();
            addView(dataRecord);        
        
            try{
                dataRecord.setPeriodFile(outputfile);
                dataRecord.setRunFile(outputfile2);
            }
            catch (IOException e) { System.out.println(e); }
       
	    }
	    
	    
	    //SET UP VIEWS
	    if (viewsOn) {
	        chartView = new ChartView();
	        agents.addView(chartView);
	        chartView.setName("Types of Agent");
	    
	        chartView.addSeries("Count Ethnocentrics", Color.green);
	        chartView.addSeries("Count DefectAgainstAll", Color.magenta);
	    
	        overheadView = new Overhead2DView();
            overheadView.setCellSize(10);
            overheadView.setName("Agent Lattice View");
            lattice.addView(overheadView);	
        
        
            //This code sets up the coloring of the agents, allowing for a
            //"light" and "dark" variant of each color
            final ColorFeatureGradiated playerColor = new ColorFeatureGradiated("PlayerWealth");
            playerColor.setDataPoint(new UnitIntervalDataPoint() {
                public double getValue(Object object) {
                    
                    double foo = 0.0;
                  
                    //REDS
                    if (((EthnoPlayer) object).tag == 1) {
                            playerColor.setMaximumColor(Color.red);
    	                    playerColor.setMinimumColor(Color.white);
        	                
    	                    if (((EthnoPlayer) object).strategyToOwn) {
        	                    foo = 1.0;
    	                    }
    	                    else {
        	                    foo = 0.5;
    	                    }
        	                
    	            }
    	            
    	            //BLUES
    	            if (((EthnoPlayer) object).tag == 0) {
                            playerColor.setMaximumColor(Color.blue);
    	                    playerColor.setMinimumColor(Color.white);
        	                
    	                    if (((EthnoPlayer) object).strategyToOwn) {
        	                    foo = 1.0;
    	                    }
    	                    else {
        	                    foo = 0.5;
    	                    }
        	                
    	            }
    	            
    	            //TYPE 3 - GREENS or MAGENTAS
    	            if (((EthnoPlayer) object).tag == 2) {
                            /*
                            playerColor.setMaximumColor(Color.green);
    	                    playerColor.setMinimumColor(Color.white);
    	                    */
        	                
    	                    playerColor.setMaximumColor(Color.magenta);
    	                    playerColor.setMinimumColor(Color.white);
        	                
    	                    if (((EthnoPlayer) object).strategyToOwn) {
        	                    foo = 1.0;
    	                    }
    	                    else {
        	                    foo = 0.5;
    	                    }
        	                
    	            }
    	            
    	            //TYPE 4 - YELLOWS or BROWNS
    	            if (((EthnoPlayer) object).tag == 3) {
        	            
        	            
    	                    //orig
    	                    /*
                            playerColor.setMaximumColor(Color.yellow);
    	                    playerColor.setMinimumColor(Color.white);
    	                    */
        	                
    	                    //v3
    	                    
    	                    playerColor.setMaximumColor(Color.yellow);
    	                    playerColor.setMinimumColor(Color.white);
    	                    
        	                
    	                    //v2
        	                
    	                    //Color brown = new Color(1, 1, 0);
        	                /*
    	                    playerColor.setMaximumColor(brown);
    	                    playerColor.setMinimumColor(Color.white);
        	                */
        	                
    	                    if (((EthnoPlayer) object).strategyToOwn) {
        	                    foo = 1.0;
    	                    }
    	                    else {
        	                    //v3
    	                        //foo = 0.15;
    	                        //orig
    	                        foo = 0.2;
    	                    }
        	                
    	            }    
                    return (foo);
                }
	        });
    	   
    	   
	    //so, empty cells are light gray
	    //dark dots on dark gray backgrounds are ETHNOCENTRIC
	    //light dots on dark backgrounds are DEFECT WITH ALL
	    //dark dots on light background are UNIVERSAL COOPERATION
	    //light dots on light background are ANTI-ETHNOCENTRIC
    	   
    	   
    	   //This code sets up background cell colors, allowing for
    	   //"light" and "dark" backgrounds
	        final ColorFeatureGradiated cellColor = new ColorFeatureGradiated("PlayerStratVOther");
            cellColor.setDataPoint(new UnitIntervalDataPoint() {
                public double getValue(Object object) {
                    
                    double foo = 0.0;
                    
                    
                    if (((HostCell) object).isAvailable()) {
                        //cellColor.setMaximumColor(Color.black);
                        cellColor.setMaximumColor(Color.lightGray);
                        foo = 1.0; //EMPTY cells are light Gray
                    }
                    else {
                        
                        
                        cellColor.setMaximumColor(Color.lightGray); //dark Gray backgrounds
                        cellColor.setMinimumColor(Color.darkGray); //lightGray backgrounds
                    
                        EthnoPlayer occupant = ((EthnoPlayer) ((HostCell) object).getOccupant());
                                   
                        if (occupant.strategyToOther) {
                            foo = 1.0; //COOP WITH OTHER = light Gray
                        }
                        else {
                            foo = 0.0; //DEF WITH OTHER = dark Gray
                        }                
    	            }
        	        
                    return (foo);
                }
	        });
    	    
            overheadView.setAgentColorFeature(playerColor);
            overheadView.setCellColorFeature(cellColor);
            }//end if viewsOn
                  
            
        }//end createViews()
    
        
    public void iterateScape() {
        
        
        //1. all agents iterate (PLAY rule and FISSION rule)
	    super.iterateScape();   
	    
		//for third array position, 0=regular value, 1="shadow" value
	    boolean countMe[][] = new boolean[latticeHeight][latticeWidth];
	   	int L[][][] = new int[latticeHeight][latticeWidth][2]; //genotype at L locus (1=donate, 0=not donate)
       	int M[][][] = new int[latticeHeight][latticeWidth][2]; //genotype at M locus ("   ")
       	int Neq[][][] = new int[latticeHeight][latticeWidth][2]; //number of neighbors with same smell
       	int Nnoteq[][][] = new int[latticeHeight][latticeWidth][2]; //number of neighbors with diff smell
       	double Lprime[][][] = new double[latticeHeight][latticeWidth][2]; //avg L-locus genotype of same smell neighbors
       	double Mprime[][][] = new double[latticeHeight][latticeWidth][2]; //avg M-locus " 							 "
		int W[][] = new int[latticeHeight][latticeWidth]; //fitness
		int color[][][] = new int[latticeHeight][latticeWidth][2];
		double PTR[][] = new double[latticeHeight][latticeWidth];
		
		if ((regressionTest) && (getIteration() == stopAtIteration)) {
			regressionTestOutputBefore();
		}
		
	
	    if ((regressionData) && (getIteration() == stopAtIteration)) {
			//walk through agents
			for (int h=0; h<latticeHeight; h++) {
				for (int j=0; j<latticeWidth; j++) {
				
				Coordinate2DDiscrete pos = new Coordinate2DDiscrete(h,j);

				if (lattice.getCell(pos).getOccupant() == null) {
					countMe[h][j] = false;
					W[h][j] = -99;
				}
				else {
				
				countMe[h][j] = true;

				//add indiv. info
				L[h][j][0] = ((EthnoPlayer) (lattice.getCell(pos).getOccupant())).getStrategyToOwn();			
				M[h][j][0] = ((EthnoPlayer) (lattice.getCell(pos).getOccupant())).getStrategyToOther();
				color[h][j][0] = ((EthnoPlayer) (lattice.getCell(pos).getOccupant())).getTag();
				
				L[h][j][1] = ((EthnoPlayer) (lattice.getCell(pos).getOccupant())).getShadowStrategyToOwn();			
				M[h][j][1] = ((EthnoPlayer) (lattice.getCell(pos).getOccupant())).getShadowStrategyToOther();
				color[h][j][1] = ((EthnoPlayer) (lattice.getCell(pos).getOccupant())).getShadowTag();

				PTR[h][j] = ((EthnoPlayer) (lattice.getCell(pos).getOccupant())).getPTR();

				//add neighbor info
				Cell[] neighbors = ((EthnoPlayer) (lattice.getCell(pos).getOccupant())).getNeighborsOnHost();
				int Lcounter = 0;
				int Mcounter = 0;
				int sameCounter = 0;
				int diffCounter = 0;
				int shadowLcounter = 0;
				int shadowMcounter = 0;
				int shadowSameCounter = 0;
				int shadowDiffCounter = 0;
								
				for (int k=0; k<neighbors.length; k++) {
					if (!(neighbors[k].isAvailable())) { //if neighbor not empty 

						int neighborTag = ((EthnoPlayer) neighbors[k]).getTag();
						int neighborShadowTag = ((EthnoPlayer) neighbors[k]).getShadowTag();
												
						if (neighborTag == ((EthnoPlayer) (lattice.getCell(pos).getOccupant())).getTag()) {
							Neq[h][j][0]++;
							Lcounter += ((EthnoPlayer) neighbors[k]).getStrategyToOwn();
							sameCounter++;
						}
						else {
							Nnoteq[h][j][0]++;
							Mcounter += ((EthnoPlayer) neighbors[k]).getStrategyToOther();
							diffCounter++;
						}
						
						//shadow alleles
						if (neighborShadowTag == ((EthnoPlayer) (lattice.getCell(pos).getOccupant())).getShadowTag()) {
							Neq[h][j][1]++;
							shadowLcounter += ((EthnoPlayer) neighbors[k]).getShadowStrategyToOwn();
							shadowSameCounter++;
						}
						else {
							Nnoteq[h][j][1]++;
							shadowMcounter += ((EthnoPlayer) neighbors[k]).getShadowStrategyToOther();
							shadowDiffCounter++;
						}
						
						
						
					}//end if notempty	
				}//end for int k=0
				
				if (sameCounter == 0) {
					Lprime[h][j][0] = -999.0;
				}
				else {Lprime[h][j][0] = ((double) Lcounter) / ((double) sameCounter);}
				
				if (diffCounter == 0) {
					Mprime[h][j][0] = -999.0;
				}
				else{Mprime[h][j][0] = ((double) Mcounter) / ((double) diffCounter);}
				
				W[h][j] = 1;
				
				
				//shadow alleles
				if (shadowSameCounter == 0) {
					Lprime[h][j][1] = -999.0;
				}
				else {Lprime[h][j][1] = ((double) shadowLcounter) / ((double) shadowSameCounter);}
				
				if (shadowDiffCounter == 0) {
					Mprime[h][j][1] = -999.0;
				}
				else{Mprime[h][j][1] = ((double) shadowMcounter) / ((double) shadowDiffCounter);}
				
				
				}//end else
							
			}//end for int j
		}//end for int h
	}//end if regressionData&StopatIteration
	
	    	    
	    //2. implement random death routine to bring pop back to max capacity
		int maxSize = ((int) ((1.0-vacancyProportion)*(latticeHeight*latticeWidth)));
		while (agents.getSize() > maxSize) {
			EthnoPlayer candidate = ((EthnoPlayer) agents.findRandomCell());
			
			if ((!(candidate.getJustBorn())) && (getIteration() == stopAtIteration)){
				Coordinate2DDiscrete coords = ((Coordinate2DDiscrete) candidate.getHostCell().getCoordinate());
				W[coords.getXValue()][coords.getYValue()]--;
			}
			
			candidate.die();
			
		}	    
	    
	    
	    //3. place surviving offspring in closest possible vacant cell to parent
	    
	    //implementation of "city block" placement
	 	
	 	//a. create array with list of all agents
	 	Agent[] allAgents = new Agent[agents.getSize()];
		allAgents = agents.getAgents();
		
		//b. create empty scapevector to fill with list of unplaced
		Vector unplacedClones = new Vector();
		
		//c. walk through full array, adding any unplaced agents to list of unplaced
	 	for (int i = 0; i<allAgents.length; i++) {
			EthnoPlayer currAgent = ((EthnoPlayer) allAgents[i]);
			if (currAgent.getIsUnplaced()) {unplacedClones.addElement(currAgent);}		
	    }
	    
	    
	    //d. place everyone on the list as follows:
	 
	 	if (locateChildNearParent) {
	    
	    boolean done = false;
	    int distance = -1;
	    boolean distanceNdone = false;
	    
	    while (!done) {
	    	if (unplacedClones.size() > 0) {
	 		
	    	distance++;
	 
	 		if (verbosePlacement) {
	    	System.out.print("* DISTANCE=	");
	    	System.out.print(distance);
	    	
	    	System.out.print(", UNPLACED SIZE=");
	 		System.out.println(unplacedClones.size());
	 		}
	    	
	    	//create temp vector which is a copy of the list of currently unplaced clones	
	    	Vector distanceNagents = ((Vector) unplacedClones.clone());
			
			distanceNdone = false;	
	    	
	    	while (!distanceNdone) {
	 
	 			if (distanceNagents.size() > 0) {
	 				
	 			
	    		//choose an agent at random from the temp vector
	    		
	    		EthnoPlayer theAgent = ((EthnoPlayer) distanceNagents.get(randomInRange(0,(distanceNagents.size()-1))));
	 		
	    		//if possible to place them at this distance, do so
	    

				//i. get list of all currently available cells (realtime)  				
  				HostCell[] availCells = lattice.findAvailable();
  				
  				
  				//ii. generate sublist of avail. cells within allowable distance
  				Vector availCloseCellsList = new Vector();
  				
  				for (int i=0; i<availCells.length; i++) {
  					if (lattice.getDistance((lattice.getCell(theAgent.getParentLoc())), availCells[i]) == distance) {
  						availCloseCellsList.addElement(availCells[i]);
  					}
  				}
  					      		
  				//(turn the list into an array)
				HostCell[] availCloseCells = new HostCell[availCloseCellsList.size()];
				availCloseCellsList.copyInto(availCloseCells);

			
				if (verbosePlacement) {
				System.out.print(" * totAvailcells= ");
				System.out.print(availCells.length);
				    
				System.out.print(" * availCloseCells= ");
				System.out.println(availCloseCells.length);
				}
				
				///iii. if sublist size is nonzero, place agent
	    		if (availCloseCells.length > 0) {
	    			//place them
	    			theAgent.moveTo(availCloseCells[randomInRange(0,(availCloseCells.length-1))]);
	    			theAgent.setIsUnplaced(false);
	    			//remove them from the master unplaced list
	    			unplacedClones.removeElement(theAgent);
	    			//update parent's fitness score
	    			Coordinate2DDiscrete pcoords = ((Coordinate2DDiscrete) theAgent.getParentLoc());
	    			W[pcoords.getXValue()][pcoords.getYValue()]++;
	    			}
	 			
	    		//in either case, remove them from the temp list for this distance--we tried them
	    		distanceNagents.removeElement(theAgent);	
	 		}
	 		else {distanceNdone = true;}
	    	}//end while !distanceNdone
	    }
	    else{ done = true;}
	    }//end while !done
		
		}//end if locateChildNearParent
		else { //locate offspring randomly
			
			while (unplacedClones.size()>0) {	
				EthnoPlayer theAgent = ((EthnoPlayer) unplacedClones.get(randomInRange(0,(unplacedClones.size()-1))));
                theAgent.moveTo((HostCell) lattice.findRandomUnoccupiedCell());
	    		theAgent.setIsUnplaced(false);
	    		//remove them from the master unplaced list
	    		unplacedClones.removeElement(theAgent);
	    		//update parent's fitness score
	    		Coordinate2DDiscrete pcoords = ((Coordinate2DDiscrete) theAgent.getParentLoc());
	    		W[pcoords.getXValue()][pcoords.getYValue()]++;
	    	}
		}		

	    //4. if in last 100 iterations, update appropriate stats
	    if (getIteration() > (stopAtIteration-100)) {
	        lasthundredGenotypeStats[0] += countCoopBoth.getDataSeries("Count").getValue();
	        lasthundredGenotypeStats[1] += countCoopOwnDefOther.getDataSeries("Count").getValue();
	        lasthundredGenotypeStats[2] += countDefOwnCoopOther.getDataSeries("Count").getValue();
	        lasthundredGenotypeStats[3] += countDefBoth.getDataSeries("Count").getValue();
	    }

		if ((regressionTest) && (getIteration() == stopAtIteration)) {
			
			regressionTestOutputAfter();	
			
		}
	
		//data for regressions
		if (regressionData) {
        
        if (getIteration() == (stopAtIteration)) {
        
        
        	//output data to file
        	
        	 try {
      	        FileOutputStream fos = new FileOutputStream(regressionDataFile+runNumber+".txt", true);		  
                PrintWriter out = new PrintWriter(fos);

				out.print("Agent Index ");
				out.print(delimiter);
				out.print("Color");
				out.print(delimiter);
				out.print("PTR");
				out.print(delimiter);
				out.print("L");
                out.print(delimiter);
                out.print("M");
                out.print(delimiter);
                out.print("Neq");
                out.print(delimiter);
                out.print("Nnoteq");
                out.print(delimiter);
                out.print("Lprime");
                out.print(delimiter);
                out.print("Mprime");
                out.print(delimiter);
                out.print("W");
                
                out.print(delimiter);
				out.print("ShadowColor");
				out.print(delimiter);
				out.print("ShadowL");
                out.print(delimiter);
                out.print("ShadowM");
                out.print(delimiter);
                out.print("ShadowNeq");
                out.print(delimiter);
                out.print("ShadowNnoteq");
                out.print(delimiter);
                out.print("ShadowLprime");
                out.print(delimiter);
                out.print("ShadowMprime");
                
                out.println();

				int index = 1;
				
                for (int y=0; y<latticeHeight; y++) {
                	for (int z=0; z<latticeWidth; z++) {
                
                		if (countMe[y][z]) {
                			out.print(index);
                			out.print(delimiter);
                			out.print(color[y][z][0]);
                			out.print(delimiter);
                			out.print(PTR[y][z]);
                			out.print(delimiter);
                			out.print(L[y][z][0]);
                			out.print(delimiter);
	                		out.print(M[y][z][0]);
                			out.print(delimiter);
                			out.print(Neq[y][z][0]);
                			out.print(delimiter);
                			out.print(Nnoteq[y][z][0]);
                			out.print(delimiter);
                			out.print(Lprime[y][z][0]);
                			out.print(delimiter);
                			out.print(Mprime[y][z][0]);
                			out.print(delimiter);
                			out.print(W[y][z]);

                			out.print(delimiter);
                			out.print(color[y][z][1]);
                			out.print(delimiter);
                			out.print(L[y][z][1]);
                			out.print(delimiter);
	                		out.print(M[y][z][1]);
                			out.print(delimiter);
                			out.print(Neq[y][z][1]);
                			out.print(delimiter);
                			out.print(Nnoteq[y][z][1]);
                			out.print(delimiter);
                			out.print(Lprime[y][z][1]);
                			out.print(delimiter);
                			out.print(Mprime[y][z][1]);


                			out.println();
                			index++;
                		}
                	}
                }
                
	            out.println();
	            
	            out.close();
	     
        	    } catch (FileNotFoundException e) {
		        	System.err.println("FileStreamsTest: "+ e);
	        	} catch (IOException e) {
			        System.err.println("FileStreamsTest: "+ e);
		        }	
 
        
		}
		}//end if regressionData
	    
	    
	    if (founderDataOutput) {
	    
	    if (getIteration() == stopAtIteration) {
	    	
	    	
	    	Agent[] theAgentsList = new Agent[agents.getSize()];
			theAgentsList = agents.getAgents();
		
		
		 try {
      	        FileOutputStream fos = new FileOutputStream(outputPath+"founderDataEXPL"+experimentNumber+"-"+runNumber+".txt", true);		  
                PrintWriter out = new PrintWriter(fos);

	    
	    		for (int j=0; j<theAgentsList.length; j++) {
			
					out.print(((EthnoPlayer) theAgentsList[j]).founderXLoc);
					out.print(",");
					out.println(((EthnoPlayer) theAgentsList[j]).founderYLoc);
			
				}
	    
	    
	            
	            out.println();
	            
	            out.close();
	     
        	    } catch (FileNotFoundException e) {
		        	System.err.println("FileStreamsTest: "+ e);
	        	} catch (IOException e) {
			        System.err.println("FileStreamsTest: "+ e);
		        }	
 		

	    }	
	    		
	    }//end if founderDataOutput
	    
	    
	    //summary data output (last iteration)	    
	    if (summaryDataOutput) {
	    
	    if (getIteration() == (stopAtIteration)) {
	    
	      //last iteration genotype stats
	      //(not used in current version, but collected in case useful later)
          double dataCoopBoth = countCoopBoth.getDataSeries("Count").getValue();
          double dataCoopOwnDefOther = countCoopOwnDefOther.getDataSeries("Count").getValue();
          double dataDefOwnCoopOther = countDefOwnCoopOther.getDataSeries("Count").getValue();
          double dataDefBoth = countDefBoth.getDataSeries("Count").getValue();

          //last iteration color stats
          //(not used in current version, but collected in case useful later)
          double dataCountRed = countRed.getDataSeries("Count").getValue();
          double dataCountBlue = countBlue.getDataSeries("Count").getValue();
          double dataCountGreen = countGreen.getDataSeries("Count").getValue();
          double dataCountYellow = countYellow.getDataSeries("Count").getValue();
        
          int ALLC = 0;
          int ALLD = 1;
          int ETHNO = 2;
          int ANTIETHNO = 3;
        
          Agent[] theCells = lattice.getAgents();
        
          //avg. Genotypes over last 100 iterations
          for (int i=0;i<4;i++) {
            lasthundredGenotypeStats[i] = lasthundredGenotypeStats[i]/100;
          }

		  if (runNumber == 1) {
			
            try {
    	        FileOutputStream fos = new FileOutputStream(summaryDataFile, true);		  
                PrintWriter out = new PrintWriter(fos);

                out.print("run#");
	            out.print(delimiter);
	            out.print("rndmSeed");
	            out.print(delimiter);
                out.print("stopIter");
                out.print(delimiter);
                
                out.print("cumulCoopOwn");
                out.print(delimiter);
                out.print("cumulDefOther");
                out.print(delimiter);
                out.print("cumulEthnoBehav");
                out.print(delimiter);
                out.print("cumulMeetOwn");
                out.print(delimiter);
                out.print("cumulTotCoop");
                out.print(delimiter);
                
                out.print("last100CoopOwn");
                out.print(delimiter);
                out.print("last100DefOther");
                out.print(delimiter);
                out.print("last100EthnoBehav");
                out.print(delimiter);
                out.print("last100MeetOwn");
                out.print(delimiter);
                out.print("last100TotCoop");
                out.print(delimiter);

				out.print("last100-C,C");
				out.print(delimiter);
	            out.print("last100-C,D");
				out.print(delimiter);
	            out.print("last100-D,C");
				out.print(delimiter);
	            out.print("last100-D,D");
				out.print(delimiter);           
	            
	            out.println();
	            
	            out.close();
	     
        	    } catch (FileNotFoundException e) {
		        	System.err.println("FileStreamsTest: "+ e);
	        	} catch (IOException e) {
			        System.err.println("FileStreamsTest: "+ e);
		        }	
		  	
		  	
		  	
		  }
        
        
          //This code specifies the output syntax for the summary data
          try {
      	        FileOutputStream fos = new FileOutputStream(summaryDataFile, true);		  
                PrintWriter out = new PrintWriter(fos);

                out.print(runNumber);
	            out.print("     ");

	            out.print(getRandomSeed());
	            out.print("     ");

                out.print(getIteration());
                out.print("     ");
                
                //CUMULATIVE OVER RUN
                //% COOPOWN
                out.print((double) ((double) phenotypeStats[COOPOWN]/(double) phenotypeStats[MEETOWN])); //% coopOwn
                out.print("     ");
                
                //% DEFOTHER
                out.print((double) ((double) phenotypeStats[DEFOTHER]/(double) phenotypeStats[MEETOTHER])); //% defOther
                out.print("     ");
                
                //% consisEthno
                out.print((double) (((double) phenotypeStats[DEFOTHER]) + ((double) phenotypeStats[COOPOWN])) / (((double) phenotypeStats[MEETOWN]) + ((double) phenotypeStats[MEETOTHER])));
                out.print("     ");
                
                //% meetOwn
                out.print((double) (((double) phenotypeStats[MEETOWN])/(((double) phenotypeStats[MEETOWN]) + ((double) phenotypeStats[MEETOTHER]))));
                out.print("     ");
                
                //% C
                out.print((double) (((double) phenotypeStats[COOPOWN]) + ((double) phenotypeStats[COOPOTHER])) / (((double) phenotypeStats[MEETOWN]) + ((double) phenotypeStats[MEETOTHER])));
                out.print("     ");
                
           
                //LASTHUNDRED
                //% COOPOWN lastHundred
                out.print((double) ((double) lasthundredPhenotypeStats[COOPOWN]/(double) lasthundredPhenotypeStats[MEETOWN])); //% coopOwn
                out.print("     ");
                
                //% DEFOTHER lastHundred
                out.print((double) ((double) lasthundredPhenotypeStats[DEFOTHER]/(double) lasthundredPhenotypeStats[MEETOTHER])); //% defOther
                out.print("     ");
                
                //% consisEthno lastHundred
                out.print((double) (((double) lasthundredPhenotypeStats[DEFOTHER]) + ((double) lasthundredPhenotypeStats[COOPOWN])) / (((double) lasthundredPhenotypeStats[MEETOWN]) + ((double) lasthundredPhenotypeStats[MEETOTHER])));
                out.print("     ");
                
                //% meetOwn
                out.print((double) ((double) lasthundredPhenotypeStats[MEETOWN]/((double) lasthundredPhenotypeStats[MEETOWN] + (double) lasthundredPhenotypeStats[MEETOTHER])));
                out.print("     ");
                
                //% C
                out.print((double) ((double) lasthundredPhenotypeStats[COOPOWN] + (double) lasthundredPhenotypeStats[COOPOTHER]) / ((double) lasthundredPhenotypeStats[MEETOWN] + (double) lasthundredPhenotypeStats[MEETOTHER]));
                out.print("     ");
                
                
	            //Genotypes over 100 final periods
	            for (int i=0;i<4;i++){
	                out.print(lasthundredGenotypeStats[i]);
	                out.print("     ");
	            }     
	            
	            
	            out.println();
	            
	            out.close();
	     
        	    } catch (FileNotFoundException e) {
		        	System.err.println("FileStreamsTest: "+ e);
	        	} catch (IOException e) {
			        System.err.println("FileStreamsTest: "+ e);
		        }	

		  
		}
              
        }
        	
	}
	
	    
	//SOME GETTERS and SETTERS for various switches
	//(These makes the global settings accessible from each player's code)
	
	public boolean getOneTagOnly() {
	    return useOneTagOnly;
	}

    public int getStopIteration() {
        return stopAtIteration;
    }

	public boolean getLocateChildNear() {
	    return locateChildNearParent;
	}
	
	public void incrementPhenotype(int _type) {
	    phenotypeStats[_type]++;
	    if (getIteration() >= (stopAtIteration-100)) {
	        lasthundredPhenotypeStats[_type]++;
	    }
	}
	
    public int getNumPlayers() {
        return agents.getSize();
    }

    public int getChildNearParentZeroFalse() {
        return (childNearParentZeroFalse);
    }
    public void setChildNearParentZeroFalse(int _value) {
        childNearParentZeroFalse = _value;
    }
    
    //part of the initialization routine each run
    public void onSetup() {
        
        super.onSetup();
        
        //test if the model has run enough times for the current
        //experiment... if so, exit
        
        if (runNumber == runMax) {
              scape.exit();
        }
        else { //otherwise...
        
        //Set up new empty vector to hold agents as they are added
		//((ScapeVector) agents).setExtent(new Coordinate1DDiscrete(0));
		//((ScapeVector) agents).setExtent(new Coordinate1DDiscrete(latticeWidth*latticeHeight));
		((ScapeVector) agents).setExtent(new Coordinate1DDiscrete(((int) ((1.0-vacancyProportion)*(latticeWidth*latticeHeight)))));
		
		
		if (colorSetting == 2) {
			useMoreColors = false;
		}
		else {
			useMoreColors = true;
		}
		
		if (colorSetting == 1) {
			useOneTagOnly = true;
		}
		else {
			useOneTagOnly = false;
		}
		
		
		//Initialize "clone near parent" switch
		if (childNearParentZeroFalse == 0) {
		    locateChildNearParent = false;
		}
		else {
		    locateChildNearParent = true;
		}
		
		//Reset stats
		for (int i=0;i<phenotypeStats.length;i++) {
	        phenotypeStats[i] = 0;
	        lasthundredPhenotypeStats[i] = 0;
	    }
	    
	    for (int i=0;i<4;i++) {
	        lasthundredGenotypeStats[i] = 0;
	    }
	
	    //increment the number of runs completed so far
	    runNumber++;
		
		System.out.println();
		System.out.print("RUN #");
		System.out.println(runNumber);
		System.out.println("******************************************");
	    
	    }
	}        
        
    
    //MORE GETTERS and SETTERS
    //(These allow abstraction of variables so they are passed back and forth
    // rather than being directly accessed across different pieces of the
    // program)
    
    public boolean getUseMoreColors() {
        return (useMoreColors);
    }
 
    public int getMaxAgents() {
        return ((int) ((1.0-vacancyProportion)*(latticeWidth*latticeHeight)));
    }
    
    public int getNumColors() {
    	return (colorSetting);
    }

	public boolean getUseCombat() {
		return (combatOn);	
	}

	public boolean getUseDiscrimCost() {
		return (discrimCostOn);
	}

	public double getDiscrimCost() {
		return (discrimCost);
	}

    public boolean getUseNoise() {
        return (noiseOn);
    }
    public double getNoiseLevel() {
        return (noiseLevel);
    }
    
    public void setPlayMode(int value) {
        playMode = value;
    }    
    
    public int getLatticeWidth() {
        return latticeWidth;
    }

    public void setLatticeWidth(int _latticeWidth) {
        latticeWidth = _latticeWidth;
    }

    public int getLatticeHeight() {
        return latticeHeight;
    }

    public void setLatticeHeight(int _latticeHeight) {
        latticeHeight = _latticeHeight;
    }
    public double getMutationRate() {
        return mutationRate;
    }
    public void setMutationRate(double mutationRate) {
        this.mutationRate = mutationRate;
    }
    
    /*
    public void incrementCountAfterImm() {
    	countAfterImm++;
    	
    }
    public void incrementCountAfterFission() {
    	countAfterFission++;
    }
    */
  
  /*
  	public void addToRegressionList(CoordinateDiscrete coords) {
  		agentRegressionList.addElement(coords);	
  	}
  */

	public void regressionTestOutputBefore() {
		
		
		System.out.println("BEFORE");
		
		try {
      	        FileOutputStream fos = new FileOutputStream(regressionDataFile+"INIT TEST.txt", true);		  
                PrintWriter out = new PrintWriter(fos);

						out.print("X coord");
                		out.print(delimiter);
                		out.print("Y coord");
                		out.print(delimiter);
                		out.print("Color");
                		out.print(delimiter);
                		out.print("Allele L");
                		out.print(delimiter);
                		out.println("Allele M");


                for (int y=0; y<latticeHeight; y++) {
                	for (int z=0; z<latticeWidth; z++) {
             
   						out.print(y);
						out.print(delimiter);
						out.print(z);
                		out.print(delimiter);
                		
                		Cell currCell = lattice.getCell(new Coordinate2DDiscrete(y,z));

                		if (currCell.getOccupant() == null) {
							out.print("EMPTY");
							out.print(delimiter);
							out.print("EMPTY");
							out.print(delimiter);
							out.println("EMPTY");                			
                		}
                		else {
                			EthnoPlayer currPlayer = ((EthnoPlayer) currCell.getOccupant());
                			out.print(currPlayer.getTag());
                			out.print(delimiter);
                			out.print(currPlayer.getStrategyToOwn());
                			out.print(delimiter);
                			out.println(currPlayer.getStrategyToOther());	
                		}
                	}
                }
                
	            out.println();
	            
	            out.close();
	     
        	    } catch (FileNotFoundException e) {
		        	System.err.println("FileStreamsTest: "+ e);
	        	} catch (IOException e) {
			        System.err.println("FileStreamsTest: "+ e);
		        }	
	}    

	public void regressionTestOutputAfter() {
		
		System.out.println("AFTER");
		
		try {
      	        FileOutputStream fos = new FileOutputStream(regressionDataFile+"AFTER TEST.txt", true);		  
                PrintWriter out = new PrintWriter(fos);

		        		out.print("X coord");
                		out.print(delimiter);
                		out.print("Y coord");
                		out.print(delimiter);
                		out.print("Color");
                		out.print(delimiter);
                		out.print("Allele L");
                		out.print(delimiter);
                		out.println("Allele M");

		
				
                for (int y=0; y<latticeHeight; y++) {
                	for (int z=0; z<latticeWidth; z++) {
                
                
        				out.print(y);
						out.print(delimiter);
						out.print(z);
                		out.print(delimiter);
                		
                		Cell currCell = lattice.getCell(new Coordinate2DDiscrete(y,z));

                		if (currCell.getOccupant() == null) {
							out.print("EMPTY");
							out.print(delimiter);
							out.print("EMPTY");
							out.print(delimiter);
							out.println("EMPTY");                			
                		}
                		else {
                			EthnoPlayer currPlayer = ((EthnoPlayer) currCell.getOccupant());
                			out.print(currPlayer.getTag());
                			out.print(delimiter);
                			out.print(currPlayer.getStrategyToOwn());
                			out.print(delimiter);
                			out.println(currPlayer.getStrategyToOther());	
                		}
                	}
                }
                
	            out.println();
	            
	            out.close();
	     
        	    } catch (FileNotFoundException e) {
		        	System.err.println("FileStreamsTest: "+ e);
	        	} catch (IOException e) {
			        System.err.println("FileStreamsTest: "+ e);
		        }	
	}    


    public int getColorSetting() {
    	return (colorSetting);
    }
    public void setColorSetting(int _setting) {
    	colorSetting = _setting;
    }
    
	public void setLocalInteraction(boolean _setting) {
		    localInteraction = _setting;
	}
    public boolean getLocalInteraction() {
    	return (localInteraction);
    }
    
//    public double getDeathRate() {
//        return deathRate;
//    }
//    public void setDeathRate(double _deathRate) {
//        deathRate = _deathRate;
//    }
       
    public double getBaseOTR() {
        return baseOTR;
    }
    public void setBaseOTR(double _OTR) {
        baseOTR = _OTR;
    }
       
    public double getCostOfGiving() {
        return costOfGiving;
    }
    public void setCostOfGiving(double _cost) {
        costOfGiving = _cost;
    }
    
    public double getGainOfReceiving() {
        return gainOfReceiving;
    }
    public void setGainOfReceiving(double _gain) {
        gainOfReceiving = _gain;
    }   
    
    
}